import numpy as np 
from scipy.integrate import ode
import scipy.stats as stats

def GoalShot(mass = 0.45,distance = 40,alpha = 85, v0 = 50, windDirection = 1,radius = 0.1):
    X = stats.truncnorm((-3 - 0) / 1, (3 - 0) / 1, loc=0, scale=1)
    alpha = alpha 
    v0    = v0    
    vwind = X.rvs(1)
    
    def rightside(t,r):    
        dr = np.zeros_like(r)
        # x = r[0]
        # y = r[1]
        # w = r[2]
        # z = r[3]
        
        # mx'' = -cx' 
        # my'' = -mg -cy'
        # c = 6*eta*pi*r
        
        # c = 6 * np.pi * radius * 1.8 * 10**-5 ==> Stokes c
        vx = r[2] + windDirection*np.abs(vwind)   
        vy = r[3] 
        v = np.sqrt(vx**2+vy**2)
        phi = np.arctan(vy/vx)
        c = 0.2 * 0.45 * 1.2041 * np.pi*radius**2 * v**2
        g = 9.81
        
        # x' = w
        dr[0] = r[2]
        # y' = z
        dr[1] = r[3]
        # w' = -c/m*w
        # w' = -c/m*|v|
    
        dr[2] = -c/mass*np.cos(phi)
        # z' = -g - c/m*z (stokes)
        # z' = -g - c/m*|v| 
        dr[3] = -g -c/mass*np.sin(phi)
        return(dr)
        
    dt = 0.01
    maximumHeight = 2.44 - radius
    alphaRadian = alpha/180*np.pi
    r0 = np.zeros(4)
    r0[2] = v0/3.6*np.cos(alphaRadian)
    r0[3] = v0/3.6*np.sin(alphaRadian)
    
    y = ode(rightside)
    y.set_integrator('vode',nsteps=500,method='bdf')
    y.set_initial_value(r0,0)
    timeLine = []
    x = []
    h = []
    while y.y[0] <= distance and y.y[1] >= 0:
        y.integrate(y.t+dt)
        if np.any(np.isnan(y.y[:])) or np.any(np.isinf(y.y[:])) or y.y[2]<0:
            return(h,x,timeLine,-1)        
        timeLine.append(y.t+dt)
        x.append(y.y[0])
        h.append(y.y[1])
        
    if x[-1] < distance:
        q = -1
    elif y.y[1] <= maximumHeight:
        q = y.y[1]
        if q>2.1:
            phi = np.arctan(y.y[3]/y.y[2])
            if phi <= -0.5:
                q = q + np.abs(phi)/2
            else:
                q = 0 
    else:
        q = -1
    return(h,x,timeLine,q)


if __name__ == '__main__':   
    import pandas as pd  
    from sklearn.neighbors import KNeighborsRegressor
    
    d = pd.read_csv("soccerGoal.csv", delimiter=",").values
    dataset = np.array(d)
    dataset = np.unique(dataset, axis=0)
    
    X = dataset[:,0:4]
    Y = dataset[:,4:6]
    Q = dataset[:,6]
    
    XMin = X.min(axis=0)    
    XMax = X.max(axis=0)
    X = (X - XMin) / (XMax - XMin)
    YMin = Y.min(axis=0)
    YMax = Y.max(axis=0)
    Y = (Y - YMin) / (YMax - YMin)
    
    neigh = KNeighborsRegressor(n_neighbors=5, weights='distance')
    neigh.fit(X, Y)
    
    m = float(np.round(0.25 + 0.5*np.random.rand(1),4))
    r = float(np.round(0.05 + 0.15*np.random.rand(1),4) )
    d = float(np.round(10 + 10*np.random.rand(1),3))
    sw = float(np.random.randint(0,3)-1)
    x = np.array([m,r,d,sw]).reshape(1, -1)
    
    y = neigh.predict(x)
    y = y.squeeze()
    y = np.round(y*(YMax - YMin) + YMin,4)
    v = y[0]
    a = y[1]
    (h,xachse,timeLine,q) = GoalShot(mass = m ,distance = d,alpha = a, v0 = v, windDirection = sw, radius=r)
    mystring = str(m) + ' , ' + str(r)  + ' , ' +  str(d)  + ' , ' +  str(sw)  + ' , ' + str(v) + ' , ' + str(a) + ' , ' + str(q) + '\n'
    if q > 2.1:     
        print('success: ',mystring)
    else: 
        print('failed: ',mystring)